/*
	Lourival Oliveira da Silva 07/2004
	Biblioteca de funo de suporta ao projeto de montador

*/

#ifndef asmlib_h
#define asmlib_h

#define debug(A, B) //printf(A, B);
//
#define BACKSPACE 8
#define RETURN 13
#define COMMA 44
#define DOT 46

//Valores booleanos
#define FALSE 0
#define TRUE 1

//Cdigo de erro
#define BAD_ARGS 5
#define BAD_SRC_FILE 10
#define BAD_TARGET_FILE 15

#define oplist8086count 9

enum addressmode {amNone, amRegister, amImmediate, amDirect, amRegIndirect,
                  amBaseIndex, amRegRelative, amBaseRelIndex,
                  amDirectInterSeg, amIndirectInterSeg};

typedef enum addressmode taddressmode;
typedef unsigned char bool; 

union Value {
  short int Word;
  unsigned int Reg:3;
  char Byte;
};

typedef union Value TValue;

struct Operando {
  taddressmode AddressMode;
  TValue V;
  bool Word;
};

typedef struct Operando TOperando;

/* Campo das instrues do 8086 */

struct FlatByte {
		 unsigned int b0:1;
		 unsigned int b1:1;
		 unsigned int b2:1;
		 unsigned int b3:1;
		 unsigned int b4:1;		 
		 unsigned int b5:1;
		 unsigned int b6:1;
		 unsigned int b7:1;		 
};

typedef struct FlatByte TFlatByte;

//1 Byte
struct op8086dw {
	   unsigned int w:1;
	   unsigned int d:1;
	   unsigned int code:6;
};

typedef struct op8086dw Top8086dw;

struct op8086wreg {		 
		 unsigned int reg:3;
		 unsigned int w:1;
		 unsigned int code:4;
};

typedef struct op8086wreg Top8086wreg;

union op8086code {
	  TFlatByte flat;
	  Top8086wreg wreg;
	  Top8086dw dw;
};

typedef union op8086code Top8086code;

//2 Byte
struct op8086rm {
	   unsigned int reg2:3;
	   unsigned int reg1:3;	   
	   unsigned int mod:2;		 
};

typedef struct op8086rm Top8086rm;

union op8086data1 {
	  TFlatByte flat;
	  Top8086rm mod_rm;
	  unsigned char Byte;
};

typedef union op8086data1 Top8086data1;

struct OpFormat {
    Top8086code Opcode;
    Top8086data1 data_modrm;
    char data2;
    char data3;
    char data4;
    int size;    
};

typedef struct OpFormat TOpFormat;

struct Operacao {
    char Mnemonic[7];
    TOperando Op1;
    TOperando Op2;
    TOpFormat format;
    int tipo;    
};

typedef struct Operacao TOperacao;

/* Campo das instrues do 8086 - FIM */

/*
  amRegister:    the register addressing mode;
                 mov ax, ax

  amImmediate:   the immediate addressing mode;
			     mov ax, 25

  amDirect:      the direct addressing mode;
                 mov ax, [1000]

  amRegIndirect: the indirect addressing mode;
				 mov ax, [bx]

  amBaseIndex:   the base plus index addressing mode;
                 mov ax, [si+bx]

  amRegRelative: the register relative addressing mode;
				 mov ax, [bx+100]

  amBaseRelIndex:the base relative plus index addressing mode.
*/

taddressmode GetAddessMode(char * Op);

TOperacao GetOperation(char * Mnem, const TOperando * Op1m, const TOperando * Op2m);
void GetAddressMode(char * Operando, taddressmode *AM, int *Size);
int SplitInstruction(const char * Inst, char * Mnem, char * Op1, char * Op2);
int IsRegister(const char * txt);
int CodRegistrador(const char * Reg);
void EncodeOperacao(TOperacao * Op);
TOperando * EncodeOperando(char * Txt);
void ExibeOperando(TOperando * cOp1);
void ExibeOperacao(TOperacao * Operacao);
void ExibeTipoParametro(const taddressmode am);
void ListaOperacoes(void);
void ExibeFlat(const TFlatByte* flat);
unsigned int GetModRM(const taddressmode am1, const taddressmode am2);
#endif
